#!/usr/bin/env python3
#
# Copyright 2020 Graviti. All Rights Reserved.
#

"""This file contains invalidation which may be found in the dataset."""

from typing import Any, Union

from .location import (
    DataLocationForDataset,
    DataLocationForFusionDataset,
    LabelLocation,
    SegmentLocation,
)

DataLocationType = Union[DataLocationForDataset, DataLocationForFusionDataset]


class Invalidation:
    """This class defines the invalidation."""

    def find_invalidation(self) -> None:
        """Find invalidation."""

    def __str__(self) -> str:
        return "Info: Invalidation"


class DatasetInvalidation(Invalidation):
    """This class defines the invalidation in dataset."""

    def find_invalidation(self) -> None:
        """Find invalidation."""

    def __str__(self) -> str:
        return "Info: Dataset Invalidation"


class LabelTableInvalidation(Invalidation):
    """This class defines the invalidation in label table."""

    def find_invalidation(self) -> None:
        """Find invalidation."""

    def __str__(self) -> str:
        return "Info: LabelTable Invalidation"


class SegmentInvalidation(Invalidation):
    """This class defines the invalidation in segment.

    :param location: the location of segment in dataset(either Dataset or FusionDataset)
    """

    def __init__(self, location: SegmentLocation) -> None:
        self._location = location

    def find_invalidation(self) -> None:
        """Find invalidation."""

    def __str__(self) -> str:
        return "Location: '{self._location}' Info: Segment Invalidation"


class DataInvalidation(Invalidation):
    """This class defines the invalidation in data.

    :param location: the location of data in dataset(either Dataset or FusionDataset)
    """

    def __init__(self, location: DataLocationType) -> None:
        self._location = location

    def find_invalidation(self) -> None:
        """Find invalidation."""

    def __str__(self) -> str:
        return f"Location: '{self._location}' Info: Data Invalidation"


# FileInvalidation
class FileNotExistInvalidation(DataInvalidation):
    """This class defines the invalidation
    --"Files does not exist according to the given path".

    :param location: the location of data in dataset(either Dataset or FusionDataset)
    """

    def __init__(self, location: DataLocationType) -> None:
        DataInvalidation.__init__(self, location)

    def find_invalidation(self) -> None:
        """Find invalidation."""

    def __str__(self) -> str:
        return (
            f"Location: '{self._location}' Info: Files does not exist according to the given path"
        )


# LabelInvalidation
class CategoryNotExistInvalidation(DataInvalidation):
    """This class defines the invalidation
    --"Category cannot be found in the label table".

    :param location: the location of data in dataset(either Dataset or FusionDataset)
    :param label_location: the location of label in data
    """

    def __init__(self, location: DataLocationType, label_location: LabelLocation) -> None:
        super().__init__(location)
        self._label_location = label_location

    def find_invalidation(self) -> None:
        """Find invalidation."""

    def __str__(self) -> str:
        return (
            f"Location: '{self._location} {self._label_location}' "
            f"Info: Category cannot be found in the label table"
        )


class CategoryIsNoneInvalidation(DataInvalidation):
    """This class defines the invalidation
    --"Category cannot be None".

    :param location: the location of data in dataset(either Dataset or FusionDataset)
    :param label_location: the location of label in data
    """

    def __init__(self, location: DataLocationType, label_location: LabelLocation) -> None:
        super().__init__(location)
        self._label_location = label_location

    def find_invalidation(self) -> None:
        """Find invalidation."""

    def __str__(self) -> str:
        return f"Location: '{self._location} {self._label_location}' Info: Category cannot be None"


class AttributeNotExistInvalidation(DataInvalidation):
    """This class defines the invalidation --"Attribute cannot be found in the label table".

    :param location: the location of data in dataset(either Dataset or FusionDataset)
    :param label_location: the location of label in data
    :param attribute_name: the name of attribute in label
    """

    def __init__(
        self, location: DataLocationType, label_location: LabelLocation, attribute_name: str
    ) -> None:
        super().__init__(location)
        self._label_location = label_location
        self._attribute_name = attribute_name

    def find_invalidation(self) -> None:
        """Find invalidation."""

    def __str__(self) -> str:
        return (
            f"Location: '{self._location} {self._label_location}' "
            f"Info: Attribute does not exist(attribute_name: {self._attribute_name})"
        )


class AttributeValueOutOfBoundInvalidation(DataInvalidation):
    """This class defines the invalidation
    --"Value in the attribute is out of bound".

    :param location: the location of data in dataset(either Dataset or FusionDataset)
    :param label_location: the location of label in data
    :param attribute_name: the name of attribute in label
    :param attribute_key: the key of attribute according to label table
    :param attribute_name: the value of attribute in label
    """

    def __init__(
        self,
        location: DataLocationType,
        label_location: LabelLocation,
        attribute_name: str,
        attribute_key: str,
        attribute_value: Any,
    ) -> None:
        super().__init__(location)
        self._label_location = label_location
        self._attribute_name = attribute_name
        self._attribute_key = attribute_key
        self._attribute_value = attribute_value

    def find_invalidation(self) -> None:
        """Find invalidation."""

    def __str__(self) -> str:
        return (
            f"Location: '{self._location} {self._label_location}' "
            f"Info: Value in the attribute is out of bound"
            f"(attribute_name: {self._attribute_name}, "
            f"value: {self._attribute_value}, key: {self._attribute_key})"
        )


class AttributeValueTypeInvalidation(DataInvalidation):
    """This class defines the invalidation
    --"Value in the attribute does not match with type in the label table".

    :param location: the location of data in dataset(either Dataset or FusionDataset)
    :param label_location: the location of label in data
    :param attribute_name: the name of attribute in label
    :param attribute_name: the value of attribute in label
    """

    def __init__(
        self,
        location: DataLocationType,
        label_location: LabelLocation,
        attribute_name: str,
        attribute_value: Any,
        attribute_type: Any,
    ) -> None:
        super().__init__(location)
        self._label_location = label_location
        self._attribute_name = attribute_name
        self._attribute_value = attribute_value
        self._attribute_type = attribute_type

    def find_invalidation(self) -> None:
        """Find invalidation."""

    def __str__(self) -> str:
        return (
            f"Location: '{self._location} {self._label_location}' "
            f"Info: Value in the attribute does not match with type in the label table"
            f"(attribute_name: {self._attribute_name}, "
            f"value: {self._attribute_value}, type: {self._attribute_type})"
        )



#!/usr/bin/env python3
#
# Copyright 2020 Graviti. All Rights Reserved.
#
# pylint: disable=invalid-name

"""This file define the 17 Category Flower Dataloader"""

import glob
import os

from ...dataset import Data, Dataset
from ...label import Classification, LabelType

DATASET_NAME = "Flower17"
CATEFORIES = [
    "Daffodil",
    "Snowdrop",
    "Lily Valley",
    "Bluebell",
    "Crocus",
    "Iris",
    "Tigerlily",
    "Tulip",
    "Fritillary",
    "Sunflower",
    "Daisy",
    "Colts Foot",
    "Dandelalion",
    "Cowslip",
    "Buttercup",
    "Windflower",
    "Pansy",
]


def Flower17(path: str) -> Dataset:
    """Load the 17 Category Flower Dataset to TensorBay

    :param path: the root directory of the dataset
    The file structure should be like:
    <path>
        jpg/
            image_0001.jpg
            ...

    :return: a loaded dataset
    """
    root_path = os.path.abspath(os.path.expanduser(path))

    img_files = sorted(glob.glob(os.path.join(root_path, "jpg", "*.jpg")))

    dataset = Dataset(DATASET_NAME)
    dataset.load_label_tables(os.path.join(os.path.dirname(__file__), "labeltables.json"))
    segment = dataset.create_segment()

    for img_name in img_files:
        img_data = Data(os.path.join(root_path, img_name))
        img_data.register_label(LabelType.CLASSIFICATION)

        # There are 80 images for each category
        index = int(img_name[6:11]) - 1
        img_data.append_label(Classification(category=CATEFORIES[index // 80]))
        segment.append(img_data)

    return dataset



#!/usr/bin/env python3
#
# Copyright 2020 Graviti. All Rights Reserved.
#
# pylint: disable=invalid-name

"""Dataloader of the Car Connection Picture Dataset"""

from .CarConnection import CarConnection

__all__ = ["CarConnection"]



#!/usr/bin/env python3
#
# Copyright 2020 Graviti. All Rights Reserved.
#

"""This file contains healthcheck related functions."""


from typing import (
    Any,
    Callable,
    Generator,
    Generic,
    Iterable,
    List,
    NamedTuple,
    Optional,
    Tuple,
    Type,
    TypeVar,
)

from ..label import LabelTables, LabelType
from ..label.label_table import AttributeInfo, AttributeType, CategoryInfo
from .pipeline import PipelineForIterable

T = TypeVar("T", bound="AttributeInfoError")  # pylint: disable=invalid-name

attribute_info_pipeline: PipelineForIterable[
    AttributeInfo, "AttributeInfoError"
] = PipelineForIterable()


def check_label_table(
    label_tables: LabelTables,
) -> Generator[Tuple[LabelType, "AttributeInfoError"], None, None]:
    for label_type, label_table in label_tables.items():
        if hasattr(label_table, "attributes"):
            for error in attribute_info_pipeline(label_table.attributes.values()):
                yield label_type, error


class AttributeInfoError:
    def __init__(self, name: str) -> None:
        self._name = name


class NeitherTypeNorEnumError(AttributeInfoError):
    def __str__(self) -> str:
        return f'AttributeInfo "{self._name}": Neither "type" nor "enum" field exists'


@attribute_info_pipeline.register
def check_neither_type_nor_enum(
    attribute_info: AttributeInfo,
) -> Generator[NeitherTypeNorEnumError, None, None]:
    if not attribute_info.enum and not attribute_info.attribute_type:
        yield NeitherTypeNorEnumError(attribute_info.name)


class RedundantTypeError(AttributeInfoError):
    def __str__(self) -> str:
        return f'AttributeInfo "{self._name}": "type" field is redundant when "enum" field exists'


@attribute_info_pipeline.register
def check_redundant_type(
    attribute_info: AttributeInfo,
) -> Generator[RedundantTypeError, None, None]:
    if attribute_info.enum and attribute_info.attribute_type:
        yield RedundantTypeError(attribute_info.name)


class RangeNotSupportError(AttributeInfoError):
    def __str__(self) -> str:
        return f'AttributeInfo "{self._name}": Only number type supports range'


@attribute_info_pipeline.register
def check_range_not_support(
    attribute_info: AttributeInfo,
) -> Generator[RangeNotSupportError, None, None]:
    if attribute_info.maximum is None and attribute_info.minimum is None:
        return

    if isinstance(attribute_info.attribute_type, list):
        if AttributeType.number in attribute_info.attribute_type:
            return

        if AttributeType.integer in attribute_info.attribute_type:
            return

    else:
        if attribute_info.attribute_type in (AttributeType.number, AttributeType.integer):
            return

    yield RangeNotSupportError(attribute_info.name)


class InvalidRangeError(AttributeInfoError):
    def __str__(self) -> str:
        return f'AttributeInfo "{self._name}": Maximum is not larger than minimum'


@attribute_info_pipeline.register
def check_invalid_range(attribute_info: AttributeInfo) -> Generator[InvalidRangeError, None, None]:
    if (
        attribute_info.maximum is not None
        and attribute_info.minimum is not None
        and attribute_info.maximum <= attribute_info.minimum
    ):
        yield InvalidRangeError(attribute_info.name)



#!/usr/bin/env python3
#
# Copyright 2020 Graviti. All Rights Reserved.
#

"""This file defines class DatasetBase, Dataset and FusionDataset."""

import json
from typing import Any, Dict, Sequence, Set, TypeVar, Union, overload

from typing_extensions import Literal

from ..label import AudioLabelTable, LabelTable, LabelTables, LabelType
from ..label.label_table import Tables
from ..utility import NameClass, NameSortedList
from .segment import FusionSegment, Segment

T = TypeVar("T", FusionSegment, Segment)  # pylint: disable=invalid-name


class DatasetBase(NameClass, Sequence[T]):
    """This class defines the concept of DatasetBase,
    which represents a whole dataset contains several segments.

    :param name: Name of the dataset
    :param is_continuous: Whether the data in dataset is continuous
    """

    def __init__(self, name: str, is_continuous: bool = False) -> None:
        super().__init__(name)
        self._segments: NameSortedList[T] = NameSortedList()
        self._label_tables: LabelTables = LabelTables()
        self._is_continuous = is_continuous

    @overload
    def __getitem__(self, index: int) -> T:
        ...

    @overload
    def __getitem__(self, index: slice) -> Sequence[T]:
        ...

    def __getitem__(self, index: Union[int, slice]) -> Union[Sequence[T], T]:
        return self._segments.__getitem__(index)

    def __len__(self) -> int:
        return self._segments.__len__()

    def __str__(self) -> str:
        return f"{super()} {self._segments}"

    @property
    def is_continuous(self) -> bool:
        """Check whether the data in dataset is continuous

        :return: Return `True` if the data is continuous, otherwise return `False`
        """
        return self._is_continuous

    @property
    def label_tables(self) -> LabelTables:
        """Return label tables

        :return: label tables
        """
        return self._label_tables

    def load_label_tables(self, loads: Union[Dict[str, Dict[str, Any]], str]) -> None:
        """Load label tables from json object.

        :param loads: LabelTables dict or the name of the file which contains the LabelTables dict
        """
        if isinstance(loads, str):
            with open(loads) as file:
                label_tables_dict = json.load(file)
        else:
            label_tables_dict = loads

        self._label_tables = LabelTables(loads=label_tables_dict)

    @overload
    def create_label_table(
        self,
        label_type: Literal[
            LabelType.CLASSIFICATION,
            LabelType.BOX2D,
            LabelType.BOX3D,
            LabelType.POLYGON2D,
            LabelType.POLYLINE2D,
        ],
    ) -> LabelTable:
        ...

    @overload
    def create_label_table(self, label_type: Literal[LabelType.SENTENCE]) -> AudioLabelTable:
        ...

    @overload
    def create_label_table(self, label_type: LabelType) -> Tables:
        ...

    def create_label_table(self, label_type: LabelType) -> Tables:
        """Create a new label table with given label type and add it to label tables.

        :param label_type: the label type of the label table to create

        :return: the created label table
        """

        return self._label_tables.create_label_table(label_type)

    @overload
    def get_label_table(
        self,
        label_type: Literal[
            LabelType.CLASSIFICATION,
            LabelType.BOX2D,
            LabelType.BOX3D,
            LabelType.POLYGON2D,
            LabelType.POLYLINE2D,
        ],
    ) -> LabelTable:
        ...

    @overload
    def get_label_table(self, label_type: Literal[LabelType.SENTENCE]) -> AudioLabelTable:
        ...

    @overload
    def get_label_table(self, label_type: LabelType) -> Tables:
        ...

    def get_label_table(self, label_type: LabelType) -> Tables:
        """return the label table corresponding to given LabeleType.

        :param label_type: a instance of LabelType
        :return: a label table
        """

        return self._label_tables[label_type]

    def get_label_types(self) -> Set[LabelType]:
        """return a set contains all label types.

        :return: a set of all label types
        """

        return set(self._label_tables.keys())

    def get_segment_by_name(self, name: str) -> T:
        """return the segment corresponding to given name.

        :param name: name of the segment
        :return: the segment which matches the input name
        """

        return self._segments.get_from_name(name)

    def add_segment(self, segment: T) -> None:
        """add segment to segment list.

        :param segment: a segment to be added
        """

        self._segments.add(segment)


class Dataset(DatasetBase[Segment]):
    """This class defines the concept of dataset,
    which contains a list of segments.
    """

    def create_segment(self, segment_name: str = "") -> Segment:
        """create a segment with the given name.

        :param segment_name: The name of the created segment
        :return: The created segment
        """
        segment = Segment(segment_name)
        self._segments.add(segment)
        return segment


class FusionDataset(DatasetBase[FusionSegment]):
    """This class defines the concept of multi-sensor dataset,
    which contains a list of multi-sensor segments.
    """

    def create_segment(self, segment_name: str = "") -> FusionSegment:
        """create a fusion segment with the given name.

        :param segment_name: The name of the created fusion segment
        :return: The created fusion segment
        """
        segment = FusionSegment(segment_name)
        self._segments.add(segment)
        return segment



#!/usr/bin/env python3
#
# Copyright 2020 Graviti. All Rights Reserved.
#

"""This file contains location of each sections in the dataset."""

from ..label.label import LabelType


class SegmentLocation:
    """This class defines the location of segment in dataset(either Dataset or FusionDataset).

    :param segment_index: Segment index
    """

    def __init__(
        self,
        segment_index: int,
    ) -> None:
        self._segment_index = segment_index

    @property
    def segment_index(self) -> int:
        """Return index of segment in the dataset(either Dataset or FusionDataset).

        :return: index of segment in the dataset(either Dataset or FusionDataset)
        """
        return self._segment_index

    def __str__(self) -> str:
        return f"Segment_index: '{self._segment_index}', "


class DataLocationForDataset(SegmentLocation):
    """This class defines the location of data in dataset(Dataset).

    :param segment_index: index of segment in dataset(Dataset)
    :param data_index: index of data in segment(Dataset)
    """

    def __init__(
        self,
        segment_index: int,
        data_index: int,
    ) -> None:
        super().__init__(segment_index)
        self._data_index = data_index

    @property
    def data_index(self) -> int:
        """Return index of data in segment.

        :return: index of data in segment
        """
        return self._data_index

    def __str__(self) -> str:
        return f"Segment_index: '{self._segment_index}', " f"Data_index: '{self._data_index}', "


class DataLocationForFusionDataset(SegmentLocation):
    """This class defines the location of data in dataset(FusionDataset).

    :param segment_index: index of segment in dataset(FusionDataset)
    :param frame_index: index of frame in segment
    :param sensor_name: name of sensor in segment
    """

    def __init__(
        self,
        segment_index: int,
        frame_index: int,
        sensor_name: str,
    ) -> None:
        super().__init__(segment_index)
        self._frame_index = frame_index
        self._sensor_name = sensor_name

    @property
    def frame_index(self) -> int:
        """Return index of frame in segment.

        :return: index of frame in segment
        """
        return self._frame_index

    @property
    def sensor_name(self) -> str:
        """Return name of sensor in segment.

        :return: name of sensor in segment
        """
        return self._sensor_name

    def __str__(self) -> str:
        return (
            f"Segment_index: '{self._segment_index}', "
            f"Frame_index: '{self._frame_index}', "
            f"Sensor_name: '{self._sensor_name}', "
        )


class LabelLocation:
    """This class defines the location of label in data(either Dataset or FusionDataset).

    :param label_type: label type of data
    :param label_index: index of label in for each type of label
    """

    def __init__(
        self,
        label_type: LabelType,
        label_index: int,
    ) -> None:
        self._label_type = label_type
        self._label_index = label_index

    @property
    def label_type(self) -> LabelType:
        """Return label type of data.

        :return: label type of data
        """
        return self._label_type

    @property
    def label_index(self) -> int:
        """Return index of label in for each type of label.

        :return: index of label in for each type of label
        """
        return self._label_index

    def __str__(self) -> str:
        return f"Segment_index: '{self._label_type}', Frame_index: '{self._label_index}', "



#!/usr/bin/env python3
#
# Copyright 2020 Graviti. All Rights Reserved.
#

"""This file defines class DatasetClientBase, DatasetClient, FusionDatasetClient,
SegmentClientBase, SegmentClient, FusionSegmentClient
"""

from typing import List, Optional, Tuple, TypeVar, Union

from ..dataset import Data, Frame, FusionSegment, Segment
from ..label import LabelTables
from ..sensor import Sensor
from ..utility import NameSortedDict
from .contentset import (
    ContentSegmentClient,
    ContentsetClient,
    FusionContentSegmentClient,
    FusionContentsetClient,
)
from .labelset import (
    FusionLabelSegmentClient,
    FusionLabelsetClient,
    LabelSegmentClient,
    LabelsetClient,
    LabelsetClientBase,
)
from .requests import Client

T = TypeVar("T", bound=Sensor)  # pylint: disable=invalid-name


class SegmentClientBase:
    """This class defines the concept of a segment client and some opertions on it.

    :param name: Segment name, unique for a dataset
    :param contentset_name: Dataset name
    :param contentset_id: Dataset id
    :param client: The client used for sending request to TensorBay
    """

    _content_segment_client: Union[ContentSegmentClient, FusionContentSegmentClient]
    _label_segment_client: Union[None, LabelSegmentClient, FusionLabelSegmentClient]

    def __init__(
        self,
        name: str,
        dataset_name: str,
        dataset_id: str,
        client: Client,
    ) -> None:
        self._name = name
        self._dataset_name = dataset_name
        self._dataset_id = dataset_id
        self._client = client

    def set_description(self, description: str) -> None:
        """set description of the segment client.

        :param description: Description of the segment client to upload
        """
        self._content_segment_client.upload_description(description)

    @property
    def name(self) -> str:
        """Get the name of this segment client.

        :return: The name of the segment
        """
        return self._name

    @property
    def dataset_id(self) -> str:
        """Get the dataset ID.

        :return: The dataset ID
        """
        return self._dataset_id

    @property
    def contentset_id(self) -> str:
        """Get the contentset ID.

        :return: The contentset ID
        """
        return self._content_segment_client._contentset_id  # pylint: disable=protected-access

    @property
    def labelset_id(self) -> str:
        """Get the labelset ID.

        :return: The labelset ID
        """
        if not self._label_segment_client:
            return ""

        return self._label_segment_client._labelset_id  # pylint: disable=protected-access


class SegmentClient(SegmentClientBase):
    """ContentSegmentClient has only one sensor, supporting upload_data method."""

    _content_segment_client: ContentSegmentClient
    _label_segment_client: Optional[LabelSegmentClient]

    def __init__(  # pylint: disable=too-many-arguments
        self,
        name: str,
        dataset_name: str,
        dataset_id: str,
        client: Client,
        content_segment_client: ContentSegmentClient,
        label_segment_client: Optional[LabelSegmentClient],
    ) -> None:
        super().__init__(name, dataset_name, dataset_id, client)
        self._content_segment_client = content_segment_client
        self._label_segment_client = label_segment_client

    def upload_data(self, local_path: str, remote_path: str = "") -> None:
        """Upload data with local path to the segment.

        :param local_path: The local path of the data to upload
        :param remote_path: The path to save the data in segment client
        :raises
            GASPathError: when remote_path does not follow linux style
            GASFrameError: when uploading frame has neither timestamp nor frame_index
        """

        self._content_segment_client.upload_data(local_path, remote_path)

    def upload_data_object(self, data: Data) -> None:
        """Upload data with local path in Data object to the segment.

        :param data: The data object which represents the local file to upload
        """
        self._content_segment_client.upload_data_object(data)
        if self._content_segment_client:
            self._content_segment_client.upload_data_object(data)

    def delete_data(self, remote_paths: Union[str, List[str]]) -> None:
        """Delete data with remote paths.

        :param remote_paths: A single path or a list of paths which need to deleted,
        eg: test/os1.png or [test/os1.png]
        """
        self._content_segment_client.delete_data(remote_paths)

    def list_data(self) -> List[str]:
        """List all data in a segment client.

        :return: A list of data path
        """
        if self._label_segment_client:
            return self._label_segment_client.list_data()

        return self._content_segment_client.list_data()

    def list_data_objects(self) -> List[Data]:
        """List all `Data` object in a dataset segment.

        :return: A list of `Data` object
        """
        if self._label_segment_client:
            return self._label_segment_client.list_data_objects()

        return self._content_segment_client.list_data_objects()


class FusionSegmentClient(SegmentClientBase):
    """FusionSegmentClient has multiple sensors,
    supporting upload_sensor and upload_frame method.
    """

    _content_segment_client: FusionContentSegmentClient
    _label_segment_client: Optional[FusionLabelSegmentClient]

    def __init__(  # pylint: disable=too-many-arguments
        self,
        name: str,
        dataset_name: str,
        dataset_id: str,
        client: Client,
        content_segment_client: FusionContentSegmentClient,
        label_segment_client: Optional[FusionLabelSegmentClient],
    ) -> None:
        super().__init__(name, dataset_name, dataset_id, client)
        self._content_segment_client = content_segment_client
        self._label_segment_client = label_segment_client

    def upload_sensor_object(self, sensor: T) -> None:
        """Upload sensor to the segment client.

        :param sensor: The sensor to upload
        """
        self._content_segment_client.upload_sensor_object(sensor)

    def delete_sensors(self, sensor_names: Union[str, List[str]]) -> None:
        """Delete sensors with a single name or a name list.

        :param sensor_names: A single sensor name or a list of sensor names
        """
        self._content_segment_client.delete_sensors(sensor_names)

    def list_sensors(self) -> List[str]:
        """List all sensor names in a segment client.

        :return: A list of sensor name
        """
        return self._content_segment_client.list_sensors()

    def list_sensor_objects(self) -> NameSortedDict[Sensor]:
        """List all sensors in a segment client.

        :return: A NameSortedDict of `Sensor` object
        """
        return self._content_segment_client.list_sensor_objects()

    def upload_frame_object(self, frame: Frame, frame_index: Optional[int] = None) -> None:
        """Upload frame to the segment client.

        :param frame: The frame to upload
        :param frame_index: The frame index, used for TensorBay to sort the frame
        :raises
            GASPathError: when remote_path does not follow linux style
        """

        self._content_segment_client.upload_frame_object(frame, frame_index)

        if self._label_segment_client:
            self._label_segment_client.upload_frame_object(frame)

    def list_frame_objects(self) -> List[Frame]:
        """List all frames in the segment client.

        :return: A list `Frame` object
        """
        if self._label_segment_client:
            return self._label_segment_client.list_frame_objects()

        return self._content_segment_client.list_frame_objects()


class DatasetClientBase:
    """This class defines the concept of a dataset and some operations on it.

    :param name: Dataset name
    :param dataset_id: Dataset id
    :param contentset_id: Contentset id
    :param labelset_id: Labelset id
    :param client: The client used for sending request to TensorBay
    """

    _contentset_client: Union[ContentsetClient, FusionContentsetClient]
    _labelset_client: Union[None, LabelsetClient, FusionLabelsetClient]

    def __init__(self, name: str, dataset_id: str, client: Client) -> None:
        self._name = name
        self._dataset_id = dataset_id
        self._client = client

    def set_description(self, description: str) -> None:
        """Set description of the dataset.

        :param description: description of the dataset
        """
        post_data = {
            "datasetId": self._dataset_id,
            "desc": description,
        }
        self._client.dataset_post("updateDataset", post_data)

    @property
    def dataset_id(self) -> str:
        """Get the dataset ID.

        :return: The dataset ID
        """
        return self._dataset_id

    @property
    def contentset_id(self) -> str:
        """Get the contentset ID.

        :return: The contentset ID
        """
        return self._contentset_client._contentset_id  # pylint: disable=protected-access

    @property
    def labelset_id(self) -> str:
        """Get the labelset ID.

        :return: The labelset ID
        """
        if not self._labelset_client:
            return ""
        return self._labelset_client._labelset_id  # pylint: disable=protected-access

    def _commit(self, message: str, tag: str) -> Tuple[str, str, str]:
        post_data = {
            "datasetId": self._dataset_id,
            "versionDesc": message,
            "version": tag,
        }

        response = self._client.dataset_post("publishDataset", post_data)
        return (
            response["id"],
            response["contentSetId"],
            response["labelSetId"],
        )

    def list_segments(self) -> List[str]:
        """List all segment names in a dataset.

        :return: A list of segment names
        """
        return self._contentset_client.list_segments()

    def delete_segments(
        self,
        segment_names: Union[str, List[str]],
        force_delete: bool = False,
    ) -> None:
        """Delete segments according to the name list.

        :param name: A single segment Name or a list of the segment names, if empty, the default
        segment will be deleted, if you want to delete all segment, "_all" should be submitted.
        :param force_delete: By default, only segment with no sensor can be deleted.
        If force_delete is true, then sensor and its objects will also be deleted
        """
        self._contentset_client.delete_segments(segment_names, force_delete)

    def _create_labelset(self, label_tables: LabelTables) -> str:
        metadata = LabelsetClientBase.create_metadata(label_tables)
        if not metadata:
            raise TypeError("Empty label table")

        post_data = {
            "datasetId": self._dataset_id,
            "contentSetId": self.contentset_id,
            "type": LabelsetClientBase.TYPE_GROUND_TRUTH,
            "version": "v1.0.2",
            "meta": metadata,
        }

        data = self._client.labelset_post("createLabelSet", post_data)
        return data["labelSetId"]  # type: ignore[no-any-return]


class DatasetClient(DatasetClientBase):
    """dataset has only one sensor, supporting create segment."""

    _contentset_client: ContentsetClient
    _labelset_client: Optional[LabelsetClient]

    def __init__(  # pylint: disable=too-many-arguments
        self, name: str, dataset_id: str, client: Client, contentset_id: str, labelset_id: str
    ) -> None:
        super().__init__(name, dataset_id, client)
        self._contentset_client = ContentsetClient(name, contentset_id, client)
        if labelset_id:
            self._labelset_client = LabelsetClient(labelset_id, name, client)
        else:
            self._labelset_client = None

    def get_or_create_segment(self, name: str = "") -> SegmentClient:
        """Create a segment set according to its name.

        :param name: Segment name, can not be "_default"
        :return: Created segment with given name, if not given name, Created default segment
        """
        content_segment_client = self._contentset_client.get_or_create_segment(name)
        label_segment_client = (
            self._labelset_client.get_segment(name) if self._labelset_client else None
        )

        return SegmentClient(
            name,
            self._name,
            self._dataset_id,
            self._client,
            content_segment_client,
            label_segment_client,
        )

    def get_segment(self, name: str = "") -> SegmentClient:
        """Get a segment according to its name.

        :param name: The name of the desired segment
        :raises GASSegmentError: When the required segment does not exist
        :return: The desired segment
        """
        content_segment_client = self._contentset_client.get_segment(name)
        label_segment_client = (
            self._labelset_client.get_segment(name) if self._labelset_client else None
        )
        return SegmentClient(
            name,
            self._name,
            self._dataset_id,
            self._client,
            content_segment_client,
            label_segment_client,
        )

    def get_segment_object(self, name: str = "") -> Segment:
        """Get a segment object according to its name.

        :param name: The name of the desired segment object
        :raises GASSegmentError: When the required segment does not exist
        :return: The desired segment object
        """
        segment_client = self.get_segment(name)
        segment = Segment(name)
        for data in segment_client.list_data_objects():
            segment.append(data)

        return segment

    def upload_segment_object(
        self,
        segment: Segment,
        *,
        jobs: int = 1,
        skip_uploaded_files: bool = False,
    ) -> SegmentClient:
        """Upload a `Segment` to the dataset,
        This function will upload all info contains in the input `Segment` object,
        which includes:
        - Create a segment using the name of input `Segment`
        - Upload all `Data` in the Segment to the dataset

        :param segment: The `Segment` object contains the information needs to be upload
        :param jobs: The number of the max workers in multithread upload
        :param skip_uploaded_files: Set it to `True` to skip the uploaded files
        :return: The client used for uploading the data in the `Segment`
        """
        content_segment_client = self._contentset_client.upload_segment_object(
            segment,
            jobs=jobs,
            skip_uploaded_files=skip_uploaded_files,
        )
        label_segment_client: Optional[LabelSegmentClient] = None
        if self._labelset_client:
            label_segment_client = self._labelset_client.upload_segment_object(
                segment,
                jobs=jobs,
                skip_uploaded_files=skip_uploaded_files,
            )

        return SegmentClient(
            segment.name,
            self._name,
            self._dataset_id,
            self._client,
            content_segment_client,
            label_segment_client,
        )

    def upload_label_tables(self, label_tables: LabelTables) -> None:
        """Upload a LabelTables to the dataset,

        :param label_tables: The `LabelTables` object to upload
        """
        if not self._labelset_client:
            labelset_id = self._create_labelset(label_tables)
            self._labelset_client = LabelsetClient(labelset_id, self._name, self._client)
            return

        self._labelset_client.upload_label_tables(label_tables)

    def commit(self, message: str, tag: str) -> "DatasetClient":
        """
        Commit dataset

        :param message: The commit message of this commit
        :param tag: set a tag for current commit
        :return: The committed dataset client
        """
        dataset_id, contentset_id, labelset_id = self._commit(message, tag)
        return DatasetClient(self._name, dataset_id, self._client, contentset_id, labelset_id)


class FusionDatasetClient(DatasetClientBase):
    """Client for fusion dataset which has multiple sensors,
    supporting create fusion segment.
    """

    _contentset_client: FusionContentsetClient
    _labelset_client: Optional[FusionLabelsetClient]

    def __init__(  # pylint: disable=too-many-arguments
        self, name: str, dataset_id: str, client: Client, contentset_id: str, labelset_id: str
    ) -> None:
        super().__init__(name, dataset_id, client)
        self._contentset_client = FusionContentsetClient(name, contentset_id, client)
        if labelset_id:
            self._labelset_client = FusionLabelsetClient(labelset_id, name, client)
        else:
            self._labelset_client = None

    def get_or_create_segment(self, name: str = "") -> FusionSegmentClient:
        """Create a fusion segment set according to the given name.

        :param name: Segment name, can not be "_default"
        :return: Created fusion segment with given name, if not given name, Created default segment
        """
        content_segment_client = self._contentset_client.get_or_create_segment(name)
        label_segment_client = (
            self._labelset_client.get_segment(name) if self._labelset_client else None
        )

        return FusionSegmentClient(
            name,
            self._name,
            self._dataset_id,
            self._client,
            content_segment_client,
            label_segment_client,
        )

    def get_segment(self, name: str = "") -> FusionSegmentClient:
        """Get a fusion segment according to its name.

        :param name: The name of the desired fusion segment
        :raises GASSegmentError: When the required fusion segment does not exist
        :return: The desired fusion segment
        """
        content_segment_client = self._contentset_client.get_segment(name)
        label_segment_client = (
            self._labelset_client.get_segment(name) if self._labelset_client else None
        )
        return FusionSegmentClient(
            name,
            self._name,
            self._dataset_id,
            self._client,
            content_segment_client,
            label_segment_client,
        )

    def upload_segment_object(
        self,
        segment: FusionSegment,
        *,
        jobs: int = 1,
        skip_uploaded_files: bool = False,
    ) -> FusionSegmentClient:
        """Upload a `FusionSegment` to the dataset,
        This function will upload all info contains in the input `FusionSegment` object,
        which includes:
        - Create a segment using the name of input `FusionSegment`
        - Upload all `Sensor` in the segment to the dataset
        - Upload all `Frame` in the segment to the dataset

        :param segment: The `Segment` object needs to be uploaded
        :param jobs: The number of the max workers in multithread upload
        :param skip_uploaded_files: Set it to `True` to skip the uploaded files
        :return: The client used for uploading the data in the `FusionSegment`
        """
        content_segment_client = self._contentset_client.upload_segment_object(
            segment,
            jobs=jobs,
            skip_uploaded_files=skip_uploaded_files,
        )
        label_segment_client: Optional[FusionLabelSegmentClient] = None
        if self._labelset_client:
            label_segment_client = self._labelset_client.upload_segment_object(
                segment,
                jobs=jobs,
                skip_uploaded_files=skip_uploaded_files,
            )

        return FusionSegmentClient(
            segment.name,
            self._name,
            self._dataset_id,
            self._client,
            content_segment_client,
            label_segment_client,
        )

    def upload_label_tables(self, label_tables: LabelTables) -> None:
        """Upload a LabelTables to the fusion dataset,

        :param label_tables: The `LabelTables` object to upload
        """
        if not self._labelset_client:
            labelset_id = self._create_labelset(label_tables)
            self._labelset_client = FusionLabelsetClient(labelset_id, self._name, self._client)
            return

        self._labelset_client.upload_label_tables(label_tables)

    def commit(self, message: str, tag: str) -> "FusionDatasetClient":
        """
        Commit fusion dataset

        :param message: The commit message of this commit
        :param tag: set a tag for current commit
        :return: The committed fusion dataset client
        """
        dataset_id, contentset_id, labelset_id = self._commit(message, tag)
        return FusionDatasetClient(self._name, dataset_id, self._client, contentset_id, labelset_id)



#!/usr/bin/env python3
#
# Copyright 2020 Graviti. All Rights Reserved.
#

"""This file contains healthcheck related functions."""

from typing import Callable, Generator, Generic, Iterable, List, Optional, TypeVar, Union

V = TypeVar("V")  # pylint: disable=invalid-name
R = TypeVar("R")  # pylint: disable=invalid-name


class Pipeline(Generic[V, R]):
    def __init__(self) -> None:
        self._pipeline: List[Callable[[V], Generator[R, None, None]]] = []

    def register(
        self, func: Callable[[V], Generator[R, None, None]]
    ) -> Callable[[V], Generator[R, None, None]]:
        self._pipeline.append(func)
        return func

    def __call__(self, *args: V) -> Generator[R, None, None]:
        for check in self._pipeline:
            yield from check(*args)


class PipelineForIterable(Pipeline[V, R]):
    def __call__(self, args: Iterable[V]) -> Generator[R, None, None]:
        for arg in args:
            yield from super().__call__(arg)



#!/usr/bin/env python3
#
# Copyright 2020 Graviti. All Rights Reserved.
#
# pylint: disable=invalid-name

"""This file define the Car Connection Picture Dataloader"""

import os
from glob import glob
from typing import Union

from ...dataset import Data, Dataset
from ...label import Classification

DATASET_NAME = "CarConnection"

KEYS = [
    "Year",
    "MSRP",
    "Front Wheel Size (in)",
    "SAE Net Horsepower @ RPM",
    "Displacement (L)",
    "Engine Type (cylinder)",
    "Width, Max w/o mirrors (in)",
    "Height, Overall (in)",
    "Length, Overall (in)",
    "Gas Mileage (mpg)",
    "Drivetrain",
    "Passenger Capacity",
    "Passenger Doors",
    "Body Style",
]


def CarConnection(path: str) -> Dataset:
    """
    Load the Car Connection Picture Dataset to TensorBay
    :param path: the root directory of the dataset
    The file structure should be like:
    <path>
        <imagename>.jpg
        ...

    :return: a loaded dataset
    """
    root_path = os.path.abspath(os.path.expanduser(path))

    dataset = Dataset(DATASET_NAME)
    dataset.load_label_tables(os.path.join(os.path.dirname(__file__), "labeltables.json"))
    segment = dataset.create_segment()

    file_paths = glob(os.path.join(root_path, "*.jpg"))
    if len(file_paths) == 0:
        raise TypeError(f"No '.jpg' files found in {root_path}")
    file_paths.sort()

    for file_path in file_paths:
        data = Data(file_path)
        filename = os.path.basename(file_path)
        label = _extract_label_from_filename(filename)
        data.append_label(label)
        segment.append(data)

    return dataset


def _transfer_attribute_type(item: str) -> Union[int, str, None]:
    if item == "nan":
        return None
    if item.isnumeric():
        return int(item)

    return item


def _extract_label_from_filename(filename: str) -> Classification:
    make, model, *spec_values = filename.split("_")[:-1]

    attributes = dict(zip(KEYS, map(_transfer_attribute_type, spec_values)))

    category = ".".join((make, model))

    return Classification(attributes=attributes, category=category)



#!/usr/bin/env python3
#
# Copyright 2020 Graviti. All Rights Reserved.
#

"""This file contains checkhealth related functions."""

import os
from typing import Any, Callable, Dict, Generator, List, Optional, Tuple, Union

from ..label.label import Classification, Label, LabelType
from ..label.label_table import AttributeInfo, AttributeType, LabelTables
from .data import Data
from .dataset import Dataset, FusionDataset
from .invalidation import (
    AttributeNotExistInvalidation,
    AttributeValueOutOfBoundInvalidation,
    AttributeValueTypeInvalidation,
    CategoryIsNoneInvalidation,
    CategoryNotExistInvalidation,
    FileNotExistInvalidation,
)
from .location import DataLocationForDataset, DataLocationForFusionDataset, LabelLocation

DataLocationType = Union[DataLocationForDataset, DataLocationForFusionDataset]
FileReportInvalidationType = FileNotExistInvalidation
LabelReportInvalidationType = Union[
    AttributeValueTypeInvalidation,
    AttributeValueOutOfBoundInvalidation,
    AttributeNotExistInvalidation,
    CategoryIsNoneInvalidation,
    CategoryNotExistInvalidation,
]


class HealthReport:
    """This class defines a health report to a dataset in which invalidation is put"""

    def __init__(self) -> None:
        self._logging_max: int = 10
        self._file_report: List[FileReportInvalidationType] = []
        self._label_report: List[LabelReportInvalidationType] = []

    @property
    def file_report(self) -> List[FileReportInvalidationType]:
        """Return list of invalidation related to file check of the dataset
        (either Dataset or FusionDataset).

        :return: list of invalidation related to file check of the dataset
        (either Dataset or FusionDataset)
        """
        return self._file_report

    @property
    def label_report(self) -> List[LabelReportInvalidationType]:
        """Return list of invalidation related to label check of the dataset
        (either Dataset or FusionDataset).

        :return: list of invalidation related to label check of the dataset
        (either Dataset or FusionDataset)
        """
        return self._label_report

    def add_to_file_report(self, invalidation: FileReportInvalidationType) -> None:
        """Insert the invalidation into the report and
        restrict the count of logging the invalidation.

        :param invalidation: invalidation related to file check
        """
        self._file_report.append(invalidation)
        self._check_logging_count(self._file_report)

    def add_to_label_report(self, invalidation: LabelReportInvalidationType) -> None:
        """Insert the error into the report and
        restrict the count of logging the invalidation.

        :param invalidation: invalidation related to label check
        """
        self._label_report.append(invalidation)
        self._check_logging_count(self._label_report)

    def _check_logging_count(self, report: List[Any]) -> None:
        """Restrict the count of logging the invalidation.

        :param report: list of invalidation in terms of the check service which is implemented
        """
        report_length = len(report)
        if report_length <= self._logging_max:
            print(report[-1])
        if report_length == self._logging_max:
            print("more info shown in the report")


def checkhealth(dataset: Union[Dataset, FusionDataset]) -> HealthReport:
    """Check if a dataset(either Dataset or FusionDataset) is legal.

    :param dataset: Union[Dataset, FusionDataset]
    :return HealthReport
    """
    health_report = HealthReport()
    check_file_exist(dataset, health_report)
    check_labels(dataset, health_report)
    return health_report


def check_file_exist(dataset: Union[Dataset, FusionDataset], health_report: HealthReport) -> None:
    """Check if files shown in data exist.

    :param dataset: Union[Dataset, FusionDataset]
    :param health_report: HealthReport
    """
    print("- Check file existence start")

    # identify the dataset type
    func: Union[
        Callable[[Dataset], Generator[Tuple[Data, DataLocationForDataset], None, None]],
        Callable[[FusionDataset], Generator[Tuple[Data, DataLocationForFusionDataset], None, None]],
    ]
    if isinstance(dataset, Dataset):
        func = _traverse_data_in_dataset
    elif isinstance(dataset, FusionDataset):
        func = _traverse_data_in_fusion_dataset
    else:
        raise TypeError("It is not a DatasetBase type")

    # traverse data in the dataset
    data_iter = func(dataset)  # type: ignore[arg-type]
    for data, location in data_iter:
        _check_file_exist(data, health_report, location)

    # check if there is invalidation in file_report
    if len(health_report.file_report) != 0:
        print("FAIL")
    else:
        print("PASS")
    print("--------------------------")


def check_labels(dataset: Union[Dataset, FusionDataset], health_report: HealthReport) -> None:
    """Check if labels shown in data are legal.

    :param dataset: Union[Dataset, FusionDataset]
    :param health_report: HealthReport
    """
    print("- Check Labels start")

    label_tables = dataset.label_tables
    if not label_tables:
        print("No label tables")
        return

    # identify the dataset type
    func: Union[
        Callable[[Dataset], Generator[Tuple[Data, DataLocationForDataset], None, None]],
        Callable[[FusionDataset], Generator[Tuple[Data, DataLocationForFusionDataset], None, None]],
    ]
    if isinstance(dataset, Dataset):
        func = _traverse_data_in_dataset
    elif isinstance(dataset, FusionDataset):
        func = _traverse_data_in_fusion_dataset
    else:
        raise TypeError("It is not a DatasetBase type")

    # traverse data in the dataset
    data_iter = func(dataset)  # type: ignore[arg-type]
    for data, location in data_iter:
        _check_labels(data, label_tables, health_report, location)

    # check if there is invalidation in file_report
    if len(health_report.label_report) != 0:
        print("FAIL")
    else:
        print("PASS")
    print("--------------------------")


def _traverse_data_in_dataset(
    dataset: Dataset,
) -> Generator[Tuple[Data, DataLocationForDataset], None, None]:
    """Data traversal in the dataset(Dataset).

    :param dataset: Dataset
    :yield sequence of data and the location of data(Dataset)
    """
    for segment_index, segment in enumerate(dataset):
        for data_index, data in enumerate(segment):
            location = DataLocationForDataset(segment_index=segment_index, data_index=data_index)
            yield data, location


def _traverse_data_in_fusion_dataset(
    fusion_dataset: FusionDataset,
) -> Generator[Tuple[Data, DataLocationForFusionDataset], None, None]:
    """Data traversal in the dataset(FusionDataset).

    :param fusion_dataset: FusionDataset
    :yield sequence of data and the location of data(FusionDataset)
    """
    for segment_index, segment in enumerate(fusion_dataset):
        for frame_index, frame in enumerate(segment):
            for sensor_name, data in frame.items():
                location = DataLocationForFusionDataset(
                    segment_index=segment_index, frame_index=frame_index, sensor_name=sensor_name
                )
                yield data, location


def _check_file_exist(
    data: Data, health_report: HealthReport, data_location: DataLocationType
) -> None:
    """Realize the function of "check_file_exist".

    :param data: each single data in dataset(either Dataset or FusionDataset)
    :param health_report: HealthReport
    :param data_location: the location of data in dataset(diverse in different dataset)
    """
    local_path = data.local_path
    if not os.path.isfile(local_path):
        invalidation = FileNotExistInvalidation(data_location)
        health_report.add_to_file_report(invalidation)


def _check_labels(
    data: Data,
    label_tables: LabelTables,
    health_report: HealthReport,
    data_location: DataLocationType,
) -> None:
    """Realize the function of "check_labels".

    :param data: each single data in dataset(either Dataset or FusionDataset)
    :param label_tables: label tables of dataset(either Dataset or FusionDataset)
    :param health_report: HealthReport
    :param data_location: the location of data in dataset(diverse in different dataset)
    """
    for label_type, labels in data.items():
        if not isinstance(labels, Classification):
            for label_index, label in enumerate(labels):
                label_location = LabelLocation(label_type, label_index)
                _get_and_check_category_and_attributes(
                    label, label_tables, health_report, data_location, label_location
                )
        else:
            label_location = LabelLocation(label_type, 1)
            _get_and_check_category_and_attributes(
                labels, label_tables, health_report, data_location, label_location
            )


def _get_and_check_category_and_attributes(
    label: Label,
    label_tables: LabelTables,
    health_report: HealthReport,
    data_location: DataLocationType,
    label_location: LabelLocation,
) -> None:
    """Get category and attributes and check if they are legal.

    :param label: each label of data in dataset(either Dataset or FusionDataset)
    :param label_tables: label tables of dataset(either Dataset or FusionDataset)
    :param health_report: HealthReport
    :param data_location: the location of data in dataset(diverse in different dataset)
    :param label_location: the location of label in data
    """

    category = label.category
    attributes = label.attributes
    if label_location.label_type != LabelType.SENTENCE:
        _check_category_legal(category, label_tables, health_report, data_location, label_location)
    _check_attributes_legal(
        category, attributes, label_tables, health_report, data_location, label_location
    )


def _check_attributes_legal(
    category: Optional[str],
    attributes: Optional[Dict[str, Any]],
    label_tables: LabelTables,
    health_report: HealthReport,
    data_location: DataLocationType,
    label_location: LabelLocation,
) -> None:
    """Check if attributes of single label are legal.

    :param category: category of label in data
    :param attributes: attributes of label in data
    :param label_tables: label tables of dataset(either Dataset or FusionDataset)
    :param health_report: LabelType
    :param data_location: the location of data in dataset(diverse in different dataset)
    :param label_location: the location of label in data
    """
    if not attributes:
        return

    for name, value in attributes.items():
        attribute_in_label_table = label_tables[label_location.label_type].attributes.get(name)

        if not attribute_in_label_table:
            attribute_not_exist_invalidation = AttributeNotExistInvalidation(
                data_location, label_location, name
            )
            health_report.add_to_label_report(attribute_not_exist_invalidation)
            continue

        _check_attribute_parent_categories_legal(
            attribute_in_label_table, category, health_report, data_location, label_location
        )

        _check_attribute_value_legal(
            attribute_in_label_table, value, health_report, data_location, label_location
        )


def _check_attribute_parent_categories_legal(
    attribute_in_label_table: AttributeInfo,
    category: Optional[str],
    health_report: HealthReport,
    data_location: DataLocationType,
    label_location: LabelLocation,
) -> None:
    """Check if single attribute of data is legal parent_categories.

    :param attribute_in_label_table: single attribute in the label
    :param category: category of label in data
    :param health_report: HealthReport
    :param data_location: the location of data in dataset(diverse in different dataset)
    :param label_location: the location of label in data
    """
    if attribute_in_label_table.parent_categories:
        if not category:
            pass

        elif category not in attribute_in_label_table.parent_categories:
            attribute_not_exist_invalidation = AttributeNotExistInvalidation(
                data_location, label_location, attribute_in_label_table.name
            )
            health_report.add_to_label_report(attribute_not_exist_invalidation)


def _check_attribute_value_legal(
    attribute_in_label_table: AttributeInfo,
    value: Any,
    health_report: HealthReport,
    data_location: DataLocationType,
    label_location: LabelLocation,
) -> None:
    """Check if attribute is legal in term of value factors.

    :param attribute_in_label_table: single attribute in the label table
    :param value: value of attribute in data
    :param health_report: HealthReport
    :param data_location: the location of data in dataset(diverse in different dataset)
    :param label_location: the location of label in data
    """
    if not attribute_in_label_table.is_array:
        _check_simple_attribute_value_legal(
            attribute_in_label_table, value, health_report, data_location, label_location
        )
    else:
        if not isinstance(value, list):
            attribute_value_type_invalidation = AttributeValueTypeInvalidation(
                data_location, label_location, attribute_in_label_table.name, value, "not list"
            )
            health_report.add_to_label_report(attribute_value_type_invalidation)

        for each_value in value:
            _check_simple_attribute_value_legal(
                attribute_in_label_table,
                each_value,
                health_report,
                data_location,
                label_location,
            )


def _check_simple_attribute_value_legal(
    attribute_in_label_table: AttributeInfo,
    value: Any,
    health_report: HealthReport,
    data_location: DataLocationType,
    label_location: LabelLocation,
) -> None:
    """Check if simple value in attribute is legal

    :param attribute_in_label_table: single attribute in the label
    :param value: single value in the single attribute
    :param health_report: HealthReport
    :param data_location: the location of data in dataset(diverse in different dataset)
    :param label_location: the location of label in data
    """
    _check_attribute_type_legal(
        attribute_in_label_table, value, health_report, data_location, label_location
    )
    _check_attribute_enum_legal(
        attribute_in_label_table, value, health_report, data_location, label_location
    )
    _check_attribute_maximum_legal(
        attribute_in_label_table, value, health_report, data_location, label_location
    )
    _check_attribute_minimum_legal(
        attribute_in_label_table, value, health_report, data_location, label_location
    )


def _check_attribute_type_legal(
    attribute_in_label_table: AttributeInfo,
    value: Any,
    health_report: HealthReport,
    data_location: DataLocationType,
    label_location: LabelLocation,
) -> None:
    """Check if value of single attribute of data is legal in term of "type".

    :param attribute_in_label_table: single attribute in the label
    :param value: single value in the single attribute
    :param health_report: HealthReport
    :param data_location: the location of data in dataset(diverse in different dataset)
    :param label_location: the location of label in data
    """
    if attribute_in_label_table.attribute_type:
        if not isinstance(attribute_in_label_table.attribute_type, list):
            if attribute_in_label_table.attribute_type.value != AttributeType.instance:
                if not isinstance(value, attribute_in_label_table.attribute_type.value):
                    attribute_value_type_invalidation = AttributeValueTypeInvalidation(
                        data_location,
                        label_location,
                        attribute_in_label_table.name,
                        value,
                        attribute_in_label_table.attribute_type.value,
                    )
                    health_report.add_to_label_report(attribute_value_type_invalidation)
            else:
                if not isinstance(value, (str, int)):
                    attribute_value_type_invalidation = AttributeValueTypeInvalidation(
                        data_location,
                        label_location,
                        attribute_in_label_table.name,
                        value,
                        attribute_in_label_table.attribute_type.value,
                    )
                    health_report.add_to_label_report(attribute_value_type_invalidation)
        else:
            attribute_types_in_label_table: List[Any] = []
            for single_type in attribute_in_label_table.attribute_type:
                if single_type.value != "instance":
                    attribute_types_in_label_table.append(single_type.value)
                else:
                    attribute_types_in_label_table.append(str)
                    attribute_types_in_label_table.append(int)
            all_types = tuple(attribute_types_in_label_table)

            if not isinstance(value, all_types):
                attribute_value_type_invalidation = AttributeValueTypeInvalidation(
                    data_location, label_location, attribute_in_label_table.name, value, all_types
                )
                health_report.add_to_label_report(attribute_value_type_invalidation)


def _check_attribute_enum_legal(
    attribute_in_label_table: AttributeInfo,
    value: Any,
    health_report: HealthReport,
    data_location: DataLocationType,
    label_location: LabelLocation,
) -> None:
    """Check if value of single attribute of data is legal in term of "enum".

    :param attribute_in_label_table: single attribute in the label
    :param value: single value in the single attribute
    :param health_report: HealthReport
    :param data_location: the location of data in dataset(diverse in different dataset)
    :param label_location: the location of label in data
    """
    if attribute_in_label_table.enum:
        if value not in attribute_in_label_table.enum:
            attribute_value_outbound_invalidation = AttributeValueOutOfBoundInvalidation(
                data_location, label_location, attribute_in_label_table.name, "enum", value
            )
            health_report.add_to_label_report(attribute_value_outbound_invalidation)


def _check_attribute_maximum_legal(
    attribute_in_label_table: AttributeInfo,
    value: Any,
    health_report: HealthReport,
    data_location: DataLocationType,
    label_location: LabelLocation,
) -> None:
    """Check if value of single attribute of data is legal in term of "maximum".

    :param attribute_in_label_table: single attribute in the label
    :param value: single value in the single attribute
    :param health_report: HealthReport
    :param data_location: the location of data in dataset(diverse in different dataset)
    :param label_location: the location of label in data
    """
    if attribute_in_label_table.maximum:
        if value > attribute_in_label_table.maximum:
            attribute_value_outbound_invalidation = AttributeValueOutOfBoundInvalidation(
                data_location, label_location, attribute_in_label_table.name, "maximum", value
            )
            health_report.add_to_label_report(attribute_value_outbound_invalidation)


def _check_attribute_minimum_legal(
    attribute_in_label_table: AttributeInfo,
    value: Any,
    health_report: HealthReport,
    data_location: DataLocationType,
    label_location: LabelLocation,
) -> None:
    """Check if value of single attribute of data is legal in term of "minimum".

    :param attribute_in_label_table: single attribute in the label
    :param value: single value in the single attribute
    :param health_report: HealthReport
    :param data_location: the location of data in dataset(diverse in different dataset)
    :param label_location: the location of label in data
    """
    if attribute_in_label_table.minimum:
        if value < attribute_in_label_table.minimum:
            attribute_value_outbound_invalidation = AttributeValueOutOfBoundInvalidation(
                data_location, label_location, attribute_in_label_table.name, "minimum", value
            )
            health_report.add_to_label_report(attribute_value_outbound_invalidation)


def _check_category_legal(
    category: Optional[str],
    label_tables: LabelTables,
    health_report: HealthReport,
    data_location: DataLocationType,
    label_location: LabelLocation,
) -> None:
    if (category is None) and (not label_tables[label_location.label_type] is None):
        category_is_none_invalidation = CategoryIsNoneInvalidation(data_location, label_location)
        health_report.add_to_label_report(category_is_none_invalidation)
        return

    if (
        category
        not in label_tables[label_location.label_type].categories.keys()  # type: ignore[union-attr]
    ):
        category_not_exist_invalidation = CategoryNotExistInvalidation(
            data_location, label_location
        )
        health_report.add_to_label_report(category_not_exist_invalidation)



#!/usr/bin/env python3
#
# Copyright 2020 Graviti. All Rights Reserved.
#

"""OpenDataset dataloader collections."""

from .CarConnection import CarConnection
from .Flower17 import Flower17
from .LISATrafficLight import LISATrafficLight

__all__ = ["LISATrafficLight", "CarConnection", "Flower17"]



#!/usr/bin/env python3
#
# Copyright 2020 Graviti. All Rights Reserved.
#

"""This file contains healthcheck related functions."""


from typing import Union

from ..dataset import Dataset, FusionDataset
from .labeltable_check import check_label_table


class HealthReport:
    def __init__(self) -> None:
        pass


def healthcheck(dataset: Union[Dataset, FusionDataset]) -> HealthReport:
    for label_type, error in check_label_table(dataset.label_tables):
        print(label_type)
        print(error)
